"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../errors");
const spying_1 = require("../spying");
const _1 = require("./");
const matcher_1 = require("./matcher");
/**
 * Checks whether functions have performed as expected
 */
class FunctionMatcher extends matcher_1.Matcher {
    /**
     * Checks that a function throws an error when executed
     */
    toThrow() {
        let errorThrown = null;
        try {
            this.actualValue();
        }
        catch (error) {
            errorThrown = error;
        }
        if ((errorThrown === null) === this.shouldMatch) {
            throw new errors_1.ErrorMatchError(errorThrown, this.shouldMatch);
        }
    }
    toThrowAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            let errorThrown = null;
            try {
                yield this.actualValue();
            }
            catch (error) {
                errorThrown = error;
            }
            if ((errorThrown === null) === this.shouldMatch) {
                throw new errors_1.ErrorMatchError(errorThrown, this.shouldMatch);
            }
        });
    }
    /**
     * Checks that a function throws a specific error
     * @param errorType - the type of error that should be thrown
     * @param errorMessage - the message that the error should have
     */
    toThrowError(errorType, errorMessage) {
        let threwRightError = false;
        let actualError = null;
        try {
            this.actualValue();
        }
        catch (error) {
            actualError = error;
            if (error instanceof errorType && error.message === errorMessage) {
                threwRightError = true;
            }
        }
        if (threwRightError !== this.shouldMatch) {
            throw new errors_1.ErrorMatchError(actualError, this.shouldMatch, errorType, errorMessage);
        }
    }
    /**
     * Checks that a function throws a specific error asynchronously
     * @param errorType - the type of error that should be thrown
     * @param errorMessage - the message that the error should have
     */
    toThrowErrorAsync(errorType, errorMessage) {
        return __awaiter(this, void 0, void 0, function* () {
            let threwRightError = false;
            let actualError = null;
            try {
                yield this.actualValue();
            }
            catch (error) {
                actualError = error;
                if (error instanceof errorType && error.message === errorMessage) {
                    threwRightError = true;
                }
            }
            if (threwRightError !== this.shouldMatch) {
                throw new errors_1.ErrorMatchError(actualError, this.shouldMatch, errorType, errorMessage);
            }
        });
    }
    /**
     * Checks that a spy has been called
     */
    toHaveBeenCalled() {
        if (this._isFunctionSpyOrSpiedOnFunction(this.actualValue) === false) {
            throw new TypeError("toHaveBeenCalled requires value passed in to Expect to be a FunctionSpy or a spied on function.");
        }
        if ((this.actualValue.calls.length === 0) === this.shouldMatch) {
            throw new errors_1.FunctionCallMatchError(this.actualValue, this.shouldMatch);
        }
        return new _1.FunctionSpyMatcher(this.actualValue);
    }
    /**
     * Checks that a spy has been called with the specified arguments
     * @param expectedArguments - a list of arguments that the spy should have been called with
     */
    toHaveBeenCalledWith(...expectedArguments) {
        if (this._isFunctionSpyOrSpiedOnFunction(this.actualValue) === false) {
            throw new TypeError("toHaveBeenCalledWith requires value passed in to Expect to be a FunctionSpy or a spied on function.");
        }
        if (this.actualValue.calls.some((call) => this._callArgumentsMatch(call, expectedArguments)) !== this.shouldMatch) {
            throw new errors_1.FunctionCallMatchError(this.actualValue, this.shouldMatch, expectedArguments);
        }
        return new _1.FunctionSpyMatcher(this.actualValue, expectedArguments);
    }
    _callArgumentsMatch(call, expectedArguments) {
        if (call.args.length !== expectedArguments.length) {
            return false;
        }
        return call.args.every((arg, index) => {
            const expectedArgument = expectedArguments[index];
            return (arg === expectedArgument ||
                expectedArgument === spying_1.Any ||
                (expectedArgument instanceof spying_1.TypeMatcher && expectedArgument.test(arg)));
        });
    }
    _isFunctionSpyOrSpiedOnFunction(value) {
        return (value instanceof spying_1.FunctionSpy ||
            (value instanceof Function && value.calls !== undefined));
    }
}
exports.FunctionMatcher = FunctionMatcher;
//# sourceMappingURL=function-matcher.js.map