"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../errors");
const spying_1 = require("../spying");
const stringification_1 = require("../stringification");
const deep_diff_1 = require("deep-diff");
const diff_1 = require("diff");
const chalk_1 = require("chalk");
function diffString(a, b) {
    const diffs = /\s/.test(a) || /\s/.test(b) ? diff_1.diffWords(a, b) : diff_1.diffChars(a, b);
    return diffs
        .map(x => x.added
        ? chalk_1.default.green(x.value)
        : x.removed
            ? chalk_1.default.red(x.value)
            : x.value)
        .join("");
}
function buildDiff(diffs, parentProperty) {
    if (diffs.length === 0) {
        return "no differences";
    }
    else if (diffs.length === 1 && diffs[0].path === undefined) {
        const onlyDiff = diffs[0];
        if (onlyDiff.kind === "E") {
            return diffString(JSON.stringify(onlyDiff.lhs), JSON.stringify(onlyDiff.rhs));
        }
        return chalk_1.default.green(onlyDiff.lhs || onlyDiff.rhs);
    }
    // probably an array
    else if (diffs.every(x => x.path === undefined)) {
        return `[\n${diffs.map(d => buildDiff([d], `[${d.index}]`)).join("\n")}\n]`;
    }
    const depth = (parentProperty.match(/\./g) || []).length + 1;
    const padding = new Array(depth).join("  ");
    const deeperDiffs = diffs
        .filter(d => d.path && d.path.length - 1 > depth)
        .reduce((concat, d) => {
        const prop = d.path[depth];
        const value = concat[prop];
        concat[prop] = value ? [...value, d] : [d];
        return concat;
    }, {});
    return (`${padding}${diffs[0].path[depth - 1]}: {\n` +
        diffs
            .filter(d => d.path.length - 1 === depth)
            .map(d => stringifyDiffProp(d, padding + "  "))
            .join(",\n") +
        Object.keys(deeperDiffs).map(key => buildDiff(deeperDiffs[key], parentProperty + "." + key)) +
        `\n${padding}}\n`);
}
function stringifyDiffProp(diffInfo, padding) {
    if (diffInfo.kind === "N") {
        return chalk_1.default.green(`+ ${padding}${diffInfo.path[diffInfo.path.length - 1] ||
            ""}: ${JSON.stringify(diffInfo.rhs)}`);
    }
    else if (diffInfo.kind === "D") {
        return chalk_1.default.red(`- ${padding}${diffInfo.path[diffInfo.path.length - 1] ||
            ""}: ${JSON.stringify(diffInfo.lhs)}`);
    }
    else if (diffInfo.kind === "E") {
        return `  ${padding}${diffInfo.path[diffInfo.path.length - 1] ||
            ""}: "${diffString(diffInfo.lhs, diffInfo.rhs)}"`;
    }
}
/**
 * Gives functionality to ensure the outcome of a test is as expected
 */
class Matcher {
    constructor(actualValue) {
        this._shouldMatch = true;
        this._actualValue = actualValue;
    }
    get actualValue() {
        return this._actualValue;
    }
    get shouldMatch() {
        return this._shouldMatch;
    }
    /**
     * Any subsequent matcher function will be looking for the opposite criteria
     */
    get not() {
        this._shouldMatch = !this.shouldMatch;
        return this;
    }
    /**
     * Checks that a value is identical to another
     * @param expectedValue - the value that will be used to match
     */
    toBe(expectedValue) {
        this._registerMatcher((expectedValue === this._actualValue) === this.shouldMatch, `Expected ${stringification_1.stringify(this.actualValue)} ${!this.shouldMatch ? "not " : ""}` + `to be ${stringification_1.stringify(expectedValue)}.`, expectedValue);
    }
    /**
     * Checks that a value is equal to another (for objects the function will check for deep equality)
     * @param expectedValue - the value that will be used to match
     */
    toEqual(expectedValue) {
        let valueMatch;
        if (expectedValue instanceof spying_1.TypeMatcher) {
            valueMatch = expectedValue.test(this._actualValue);
        }
        else if (Buffer.isBuffer(expectedValue)) {
            valueMatch = this._checkBuffersAreEqual(expectedValue, this._actualValue);
        }
        else if (expectedValue instanceof Object) {
            valueMatch = this._checkObjectsAreDeepEqual(expectedValue, this._actualValue);
        }
        else {
            // exclude the double equals in this case from review
            // as this is what we want to do
            // tslint:disable-next-line:triple-equals
            valueMatch = expectedValue == this._actualValue;
        }
        this._registerMatcher(valueMatch === this._shouldMatch, `Expected ${stringification_1.stringify(this.actualValue)} ${!this.shouldMatch ? "not " : ""}` + `to be equal to ${stringification_1.stringify(expectedValue)}.`, expectedValue, {
            diff: this._diff(expectedValue, this._actualValue)
        });
    }
    /**
     * Checks that a value is not undefined
     */
    toBeDefined() {
        this._registerMatcher((this._actualValue !== undefined) === this.shouldMatch, `Expected ${stringification_1.stringify(this.actualValue)} ${this.shouldMatch ? "not " : ""}` + `to be undefined.`, undefined);
    }
    /**
     * Checks that a value is null
     */
    toBeNull() {
        this._registerMatcher((this._actualValue === null) === this.shouldMatch, `Expected ${stringification_1.stringify(this.actualValue)} ${!this.shouldMatch ? "not " : ""}` + `to be null.`, null);
    }
    /**
     * Checks that a value is equivalent to boolean true
     */
    toBeTruthy() {
        this._registerMatcher((this._actualValue && this.shouldMatch) ||
            (!this._actualValue && !this.shouldMatch), `Expected ${stringification_1.stringify(this.actualValue)} ${!this.shouldMatch ? "not " : ""}to be truthy.`, this.shouldMatch ? "truthy" : "falsy");
    }
    _registerMatcher(isMatch, failureMessage, expectedValue, extras) {
        if (isMatch === false) {
            throw new errors_1.MatchError(failureMessage, expectedValue, this._actualValue, extras);
        }
    }
    _diff(a, b) {
        if (typeof a === "string") {
            return diffString(a, b);
        }
        return buildDiff(deep_diff_1.diff(a, b) || [], "");
    }
    _checkBuffersAreEqual(buffer, other) {
        // Buffer.from() only accepts of type string, Buffer, ArrayBuffer, Array, or Array-like Object.
        if (this._isBufferable(other)) {
            const otherBuffer = Buffer.isBuffer(other)
                ? other
                : Buffer.from(other); // Typings don't know that Buffer.from() can accept ArrayLike<T>
            return buffer.equals(otherBuffer);
        }
        else {
            return false;
        }
    }
    _checkObjectsAreDeepEqual(objectA, objectB) {
        // if one object is an array and the other is not then they are not equal
        if (Array.isArray(objectA) !== Array.isArray(objectB)) {
            return false;
        }
        // get all the property keys for each object
        const OBJECT_A_KEYS = Object.keys(objectA);
        const OBJECT_B_KEYS = Object.keys(objectB);
        // if they don't have the same amount of properties then clearly not
        if (OBJECT_A_KEYS.length !== OBJECT_B_KEYS.length) {
            return false;
        }
        // check all the properties of each object
        for (const objectAKey of OBJECT_A_KEYS) {
            // if the property values are not the same
            if (objectA[objectAKey] !== objectB[objectAKey]) {
                // and it's not an object or the objects are not equal
                if (typeof objectA[objectAKey] !== "object" ||
                    this._checkObjectsAreDeepEqual(objectA[objectAKey], objectB[objectAKey]) === false) {
                    // then not deep equal
                    return false;
                }
            }
        }
        // all properties match so all is good
        return true;
    }
    _isBufferable(obj) {
        return ("string" === typeof obj ||
            Buffer.isBuffer(obj) ||
            Array.isArray(obj) ||
            obj instanceof ArrayBuffer ||
            // ArrayLike<any>
            (null != obj &&
                "object" === typeof obj &&
                obj.hasOwnProperty("length") &&
                "number" === typeof obj.length &&
                (obj.length === 0 || (obj.length > 0 && obj.length - 1 in obj))));
    }
}
exports.Matcher = Matcher;
//# sourceMappingURL=matcher.js.map