"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const errors_1 = require("./errors");
const results_1 = require("./results");
const stringification_1 = require("./stringification");
const js_yaml_1 = require("js-yaml");
class TestOutputStream extends stream_1.Readable {
    _read() { } // tslint:disable-line:no-empty
    end() {
        this.push(null);
    }
    emitVersion() {
        this._writeOut("TAP version 13\n");
    }
    emitPlan(testCount) {
        this._writeOut(`1..${testCount}\n`);
    }
    emitFixture(fixture) {
        this._writeOut(`# FIXTURE ${fixture.description}\n`);
    }
    emitLog(...log) {
        this._writeOut(`# LOG: ${log.join(" ")}\n`);
    }
    emitWarning(...warning) {
        this._writeOut(`# WARN: ${warning.join(" ")}\n`);
    }
    emitError(...error) {
        this._writeOut(`# ERROR: ${error.join(" ")}\n`);
    }
    emitResult(testId, result) {
        const outcome = result.outcome;
        const test = result.test;
        const testCaseArguments = result.arguments;
        if (outcome === results_1.TestOutcome.Pass) {
            this._emitPass(testId, test, testCaseArguments);
        }
        else if (outcome === results_1.TestOutcome.Fail || outcome === results_1.TestOutcome.Error) {
            this._emitFail(testId, test, testCaseArguments, result);
        }
        else if (outcome === results_1.TestOutcome.Skip) {
            this._emitSkip(testId, test, testCaseArguments);
        }
        else {
            throw new TypeError(`Invalid test outcome: ${outcome}`);
        }
    }
    _writeOut(message) {
        this.push(message);
    }
    _emitPass(testId, test, testCaseArguments) {
        const description = this._getTestDescription(test, testCaseArguments);
        this._writeOut(`ok ${testId} ${description}\n`);
    }
    _emitSkip(testId, test, testCaseArguments) {
        const description = this._getTestDescription(test, testCaseArguments);
        // we only want to use the reason if it's not undefined
        let reasonString = "";
        if (test.ignoreReason !== undefined) {
            reasonString = ` ${test.ignoreReason}`;
        }
        this._writeOut(`ok ${testId} ${description} # skip${reasonString}\n`);
    }
    _emitFail(testId, test, testCaseArguments, result) {
        const description = this._getTestDescription(test, testCaseArguments);
        this._writeOut(`not ok ${testId} ${description}\n`);
        if (result.error instanceof errors_1.MatchError) {
            this._writeMatchErrorOutput(result.error);
        }
        else {
            this._writeUnhandledErrorOutput(result.error);
        }
    }
    _getTestDescription(test, testCaseArguments) {
        if (testCaseArguments === undefined || testCaseArguments.length <= 0) {
            return test.description;
        }
        const formattedArguments = testCaseArguments.map(stringification_1.stringify).join(", ");
        return `${test.description} ( ${formattedArguments} )`;
    }
    _writeMatchErrorOutput(error) {
        const sanitisedMessage = error.message
            .replace(/\\/g, "\\\\")
            .replace(/"/g, '\\"');
        const sanitisedActual = stringification_1.stringify(error.actual);
        const sanitisedExpected = stringification_1.stringify(error.expected);
        this._writeFailure(sanitisedMessage, sanitisedActual, sanitisedExpected, error.extras);
    }
    _writeUnhandledErrorOutput(error) {
        this._writeFailure("The test threw an unhandled error.", "an unhandled error", "no unhandled errors to be thrown", error instanceof Error ? { stack: error.stack } : undefined);
    }
    _writeFailure(message, actual, expected, details) {
        const output = {
            message,
            severity: "fail",
            data: {
                got: actual,
                expect: expected,
                details
            }
        };
        if (output.data.details === undefined) {
            delete output.data.details;
        }
        this._writeOut(` ---\n${js_yaml_1.safeDump(output)
            .split("\n")
            .map(s => ` ${s}`)
            .join("\n")}...\n`);
    }
}
exports.TestOutputStream = TestOutputStream;
//# sourceMappingURL=test-output-stream.js.map