"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("./stream/stream");
const chalk_1 = require("chalk");
const parser = require("tap-parser")();
const duplexer = require("duplexer");
const { h, render, Component, Color, Indent } = require("ink");
class TapBarkOutput extends Component {
    constructor() {
        super();
        this.FIXTURE_REGEXP = /^# FIXTURE (.*)/;
        this.CONSOLE_WARNING_REGEXP = /^# WARN: (.*)/;
        this._completeCalled = false;
        this.state = {
            logs: [],
            warnings: []
        };
        this.setupListeners();
    }
    getFailureMessage(assertion) {
        const failureTitle = chalk_1.default.red("FAIL: ") + chalk_1.default.bold(assertion.name) + "\n";
        if (assertion.diag) {
            const data = assertion.diag.data;
            const details = data.details;
            const title = `${failureTitle} ${assertion.diag.message}\n`;
            if (details && Object.keys(details).length > 0) {
                return `${title}${Object.keys(details)
                    .map(key => `\n${chalk_1.default.underline(key)}:\n${details[key]}`)
                    .join("\n")}`;
            }
            return `${title}\nexpected:\n${data.expect}\nactual:\n${data.got}`;
        }
        return failureTitle + "Failure reason unknown.";
    }
    render() {
        const results = this.state.results;
        const total = this.state.totalTests;
        if (this.state.results) {
            return h(Indent, null,
                this.state.warnings.join("\n"),
                h(Color, { green: true },
                    "Pass: ",
                    results.pass,
                    " / ",
                    total,
                    "\n"),
                h(Color, { red: true },
                    "Fail: ",
                    results.fail,
                    " / ",
                    total,
                    "\n"),
                h(Color, { yellow: true },
                    "Ignore: ",
                    results.ignore,
                    " / ",
                    total,
                    "\n",
                    "\n"),
                results.failures.map(this.getFailureMessage.bind(this)).join("\n"));
        }
        return h(Indent, null,
            Math.floor(this.state.currentTest / total * 100 || 0),
            "% complete");
    }
    setupListeners() {
        parser.on("plan", (plan) => {
            this.setState({
                totalTests: plan.end
            });
        });
        // temporary while https://github.com/vadimdemedes/ink/issues/97 is still an issue
        const warnings = [];
        parser.on("comment", (comment) => {
            let fixtureParse = this.FIXTURE_REGEXP.exec(comment);
            if (fixtureParse !== null) {
                this.setState({
                    fixtureName: fixtureParse[1]
                });
            }
            else {
                const message = comment.replace("# ", "");
                if (this.CONSOLE_WARNING_REGEXP.test(comment)) {
                    // temporary while https://github.com/vadimdemedes/ink/issues/97 is still an issue
                    warnings.push(chalk_1.default.yellow(message));
                    this.setState({
                        warnings
                    });
                }
            }
        });
        parser.on("assert", (assertion) => {
            this.setState({
                currentTest: assertion.id,
                testName: assertion.name
            });
        });
        parser.on("complete", (results) => {
            let _results = {
                pass: results.pass || 0,
                fail: (results.fail || (results.failures || []).length),
                ignore: (results.skip || 0) + (results.todo || 0),
                failures: results.failures || []
            };
            // getting called multiple times for whatever reason
            if (this._completeCalled === false) {
                this._completeCalled = true;
                this.setState(Object.assign({}, this.state, { results: _results }), setTimeout(() => {
                    if (results.ok) {
                        process.exit(0);
                    }
                    else {
                        process.exit(1);
                    }
                }, 100));
            }
        });
    }
}
class TapBark {
    static create() {
        const stream = new stream_1.Stream();
        render(h(TapBarkOutput, null));
        return {
            getPipeable() {
                return duplexer(parser, stream.getUnderlyingStream());
            }
        };
    }
}
exports.TapBark = TapBark;
//# sourceMappingURL=tap-bark.js.map